@@ -1,6 +1,6 @@
 #!/usr/bin/python
 
-## Copyright (C) 2007, 2008, 2009, 2010 Red Hat, Inc.
+## Copyright (C) 2007, 2008, 2009, 2010, 2012 Red Hat, Inc.
 ## Copyright (C) 2008 Novell, Inc.
 ## Authors: Tim Waugh <twaugh@redhat.com>, Vincent Untz
 
@@ -16,16 +16,12 @@
 
 ## You should have received a copy of the GNU General Public License
 ## along with this program; if not, write to the Free Software
-## Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+## Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
 import cups
 import gobject
-import gtk
 import os
 
-import asyncipp
-import asyncpk1
-import authconn
 import config
 from debug import *
 import debug
@@ -130,6 +126,7 @@ class Connection(SemanticOperations):
 
         if use_pk and try_as_root:
             debugprint ("Using polkit-1 connection class")
+            import asyncpk1
             c = asyncpk1.PK1Connection (reply_handler=subst_reply_handler,
                                         error_handler=subst_error_handler,
                                         host=host, port=port,
@@ -138,6 +135,7 @@ class Connection(SemanticOperations):
             self._conn = c
         else:
             debugprint ("Using IPP connection class")
+            import asyncipp
             c = asyncipp.IPPAuthConnection (reply_handler=subst_reply_handler,
                                             error_handler=subst_error_handler,
                                             auth_handler=subst_auth_handler,
