@@ -16,12 +16,15 @@
 
 ## You should have received a copy of the GNU General Public License
 ## along with this program; if not, write to the Free Software
-## Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+## Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
 import cups
 import dbus
 import gobject
-import gtk
+try:
+    import gtk
+except:
+    pass
 import os
 import sys
 import tempfile
@@ -102,9 +105,15 @@ class _PK1AsyncMethodCall:
             return
 
         if str (error) == '':
-            gtk.gdk.threads_enter ()
+            try:
+                gtk.gdk.threads_enter ()
+            except:
+                pass
             self._client_reply_handler (self._conn, self._unpack_fn (*args))
-            gtk.gdk.threads_leave ()
+            try:
+                gtk.gdk.threads_leave ()
+            except:
+                pass
             self._destroy ()
             return
 
@@ -117,9 +126,15 @@ class _PK1AsyncMethodCall:
 
         if exc.get_dbus_name () == CUPS_PK_NEED_AUTH:
             exc = cups.IPPError (cups.IPP_NOT_AUTHORIZED, 'pkcancel')
-            gtk.gdk.threads_enter ()
+            try:
+                gtk.gdk.threads_enter ()
+            except:
+                pass
             self._client_error_handler (self._conn, exc)
-            gtk.gdk.threads_leave ()
+            try:
+                gtk.gdk.threads_leave ()
+            except:
+                pass
             self._destroy ()
             return
 
@@ -658,7 +673,7 @@ if __name__ == '__main__':
             print "cancel error: %s" % repr (exc)
 
         def get_file_clicked (self, button):
-            self.my_file = file ("/tmp/foo", "w")
+            self.my_file = file ("cupsd.conf", "w")
             self.conn.getFile ("/admin/conf/cupsd.conf", file=self.my_file,
                                reply_handler=self.got_file,
                                error_handler=self.get_file_error)
