@@ -1,8 +1,8 @@
-#!/usr/bin/env python
+#!/usr/bin/python
 
 ## system-config-printer
 
-## Copyright (C) 2010 Red Hat, Inc.
+## Copyright (C) 2010, 2011 Red Hat, Inc.
 ## Author: Tim Waugh <twaugh@redhat.com>
 
 ## This program is free software; you can redistribute it and/or modify
@@ -17,35 +17,64 @@
 
 ## You should have received a copy of the GNU General Public License
 ## along with this program; if not, write to the Free Software
-## Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+## Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
 import dbus
+import gobject
 import gtk
 import cupshelpers
 
+import cups
+cups.require ("1.9.52")
+
 import asyncconn
 from debug import debugprint
+from gettext import gettext as _
 
-_ = lambda x: x
-def set_gettext_function (fn):
-    global _
-    _ = fn
+class PPDsLoader(gobject.GObject):
+    """
+    1. If PackageKit support is available, and this is a local server,
+    try to use PackageKit to install relevant drivers.  We do this
+    because we can only make the right choice about the "best" driver
+    when the full complement of drivers is there to choose from.
 
-class PPDsLoader:
-    def __init__ (self, callback, device_id=None, device_uri=None, parent=None,
-                  host=None, encryption=None):
+    2. Fetch the list of available drivers from CUPS.
+
+    3. If Jockey is available, and there is no appropriate driver
+    available, try to use Jockey to install one.
+
+    4. If Jockey was able to install one, fetch the list of available
+    drivers again.
+    """
+
+    __gsignals__ = {
+        'finished': (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, [])
+        }
+
+    def __init__ (self, device_id=None, parent=None, device_uri=None,
+                  host=None, encryption=None, language=None,
+                  device_make_and_model=None):
+        gobject.GObject.__init__ (self)
         debugprint ("+%s" % self)
-        self._callback = callback
         self._device_id = device_id
         self._device_uri = device_uri
+        self._device_make_and_model = device_make_and_model
         self._parent = parent
         self._host = host
         self._encryption = encryption
+        self._language = language
 
         self._installed_files = []
         self._conn = None
         self._ppds = None
+        self._exc = None
 
+        self._ppdsmatch_result = None
+        self._jockey_queried = False
+        self._jockey_has_answered = False
+        self._local_cups = (self._host == None or
+                            self._host == "localhost" or
+                            self._host[0] == '/')
         try:
             self._bus = dbus.SessionBus ()
         except:
@@ -64,8 +93,18 @@ class PPDsLoader:
 
         self._dialog.connect ("response", self._dialog_response)
 
+    def run (self):
         self._dialog.show_all ()
-        self._query_cups (True)
+
+        if self._device_id:
+            self._devid_dict = cupshelpers.parseDeviceID (self._device_id)
+
+        if self._local_cups and self._device_id and self._bus:
+            self._gpk_device_id = "MFG:%s;MDL:%s;" % (self._devid_dict["MFG"],
+                                                      self._devid_dict["MDL"])
+            self._query_packagekit ()
+        else:
+            self._query_cups ()
 
     def __del__ (self):
         debugprint ("-%s" % self)
@@ -76,17 +115,95 @@ class PPDsLoader:
             self._dialog.destroy ()
             self._dialog = None
 
-        self._callback = None
         self._parent = None
 
+        if self._conn:
+            self._conn.destroy ()
+            self._conn = None
+
     def get_installed_files (self):
         return self._installed_files
 
     def get_ppds (self):
         return self._ppds
 
+    def get_ppdsmatch_result (self):
+        return self._ppdsmatch_result
+
+    def get_error (self):
+        return self._exc
+
     def _dialog_response (self, dialog, response):
-        self._call_callback (None)
+        dialog.destroy ()
+        self._dialog = None
+        self.emit ('finished')
+
+    def _query_cups (self):
+        debugprint ("Asking CUPS for PPDs")
+        if (not self._conn):
+            c = asyncconn.Connection (host=self._host,
+                                      encryption=self._encryption,
+                                      reply_handler=self._cups_connect_reply,
+                                      error_handler=self._cups_error)
+            self._conn = c
+        else:
+            self._cups_connect_reply(self._conn, None)
+
+    def _cups_connect_reply (self, conn, UNUSED):
+        conn._begin_operation (_("fetching PPDs"))
+        conn.getPPDs2 (reply_handler=self._cups_reply,
+                       error_handler=self._cups_error)
+
+    def _cups_reply (self, conn, result):
+        ppds = cupshelpers.ppds.PPDs (result, language=self._language)
+        self._ppds = ppds
+        self._need_requery_cups = False
+        if self._device_id:
+            fit = ppds.\
+                getPPDNamesFromDeviceID (self._devid_dict["MFG"],
+                                         self._devid_dict["MDL"],
+                                         self._devid_dict["DES"],
+                                         self._devid_dict["CMD"],
+                                         self._device_uri,
+                                         self._device_make_and_model)
+
+            ppdnamelist = ppds.\
+                orderPPDNamesByPreference (fit.keys (),
+                                           self._installed_files,
+                                           devid=self._devid_dict,
+                                           fit=fit)
+            self._ppdsmatch_result = (fit, ppdnamelist)
+
+            ppdname = ppdnamelist[0]
+            if (self._bus and
+                not fit[ppdname].startswith ("exact") and
+                not self._jockey_queried and
+                self._local_cups):
+                # Try to install packages using jockey if
+                # - there's no appropriate driver (PPD) locally available
+                # - we are configuring local CUPS server
+                self._jockey_queried = True
+                self._query_jockey ()
+                return
+
+        conn.destroy ()
+        self._conn = None
+        if self._dialog != None:
+            self._dialog.destroy ()
+            self._dialog = None
+
+        self.emit ('finished')
+
+    def _cups_error (self, conn, exc):
+        conn.destroy ()
+        self._conn = None
+        self._ppds = None
+        self._exc = exc
+        if self._dialog != None:
+            self._dialog.destroy ()
+            self._dialog = None
+
+        self.emit ('finished')
 
     def _query_packagekit (self):
         debugprint ("Asking PackageKit to install drivers")
@@ -99,8 +216,12 @@ class PPDsLoader:
             obj = self._bus.get_object ("org.freedesktop.PackageKit",
                                         "/org/freedesktop/PackageKit")
             proxy = dbus.Interface (obj, "org.freedesktop.PackageKit.Modify")
-            proxy.InstallPrinterDrivers (xid, [self._device_id],
-                                         "hide-finished",
+            resources = [self._gpk_device_id]
+            interaction = "hide-finished"
+            debugprint ("Calling InstallPrinterDrivers (%s, %s, %s)" %
+                        (repr (xid), repr (resources), repr (interaction)))
+            proxy.InstallPrinterDrivers (dbus.UInt32 (xid),
+                                         resources, interaction,
                                          reply_handler=self._packagekit_reply,
                                          error_handler=self._packagekit_error,
                                          timeout=3600)
@@ -108,19 +229,20 @@ class PPDsLoader:
             debugprint ("Failed to talk to PackageKit: %s" % e)
             if self._dialog:
                 self._dialog.show_all ()
-                self._query_jockey ()
+                self._query_cups ()
 
     def _packagekit_reply (self):
         debugprint ("Got PackageKit reply")
+        self._need_requery_cups = True
         if self._dialog:
             self._dialog.show_all ()
-            self._query_jockey ()
+            self._query_cups ()
 
     def _packagekit_error (self, exc):
         debugprint ("Got PackageKit error: %s" % exc)
         if self._dialog:
             self._dialog.show_all ()
-            self._query_jockey ()
+            self._query_cups ()
 
     def _query_jockey (self):
         debugprint ("Asking Jockey to install drivers")
@@ -136,97 +258,29 @@ class PPDsLoader:
 
     def _jockey_reply (self, conn, result):
         debugprint ("Got Jockey result: %s" % repr (result))
+        self._jockey_has_answered = True
         try:
             self._installed_files = result[1]
         except:
-            self._installed_files = ()
+            self._installed_files = []
         self._query_cups ()
 
     def _jockey_error (self, exc):
         debugprint ("Got Jockey error: %s" % exc)
-        self._query_cups ()
-
-    def _query_cups (self, local=False):
-        debugprint ("Asking CUPS for PPDs")
-        if (local):
-            c = asyncconn.Connection (host=self._host,
-                                      encryption=self._encryption,
-                                      reply_handler=self._cups_connect_reply_local,
-                                      error_handler=self._cups_error)
+        if self._need_requery_cups:
+            self._query_cups ()
         else:
-            c = asyncconn.Connection (host=self._host,
-                                      encryption=self._encryption,
-                                      reply_handler=self._cups_connect_reply,
-                                      error_handler=self._cups_error)
-        self._conn = c
+            if self._conn != None:
+                self._conn.destroy ()
+                self._conn = None
 
-    def _cups_connect_reply_local (self, conn, UNUSED):
-        if conn != self._conn:
-            conn.destroy ()
-            return
+            if self._dialog != None:
+                self._dialog.destroy ()
+                self._dialog = None
 
-        conn._begin_operation (_("fetching PPDs"))
-        conn.getPPDs (reply_handler=self._cups_reply_local,
-                      error_handler=self._cups_error)
-
-    def _cups_connect_reply (self, conn, UNUSED):
-        if conn != self._conn:
-            conn.destroy ()
-            return
-
-        conn._begin_operation (_("fetching PPDs"))
-        conn.getPPDs (reply_handler=self._cups_reply,
-                      error_handler=self._cups_error)
-
-    def _cups_reply_local (self, conn, result):
-        if conn != self._conn:
-            conn.destroy ()
-            return
-
-        conn.destroy ()
-        self._conn = None
-        self._ppds = result
-        ppds = cupshelpers.ppds.PPDs (result)
-        if self._device_id and self._bus:
-            devid_dict = cupshelpers.parseDeviceID (self._device_id)
-            (status, ppdname) = ppds.\
-                getPPDNameFromDeviceID (devid_dict["MFG"],
-                                        devid_dict["MDL"],
-                                        devid_dict["DES"],
-                                        devid_dict["CMD"],
-                                        self._device_uri,
-                                        ())
-            if status != ppds.STATUS_SUCCESS:
-                self._query_packagekit ()
-            else:
-                self._call_callback (None)
-        else:
-            self._call_callback (None)
-
-    def _cups_reply (self, conn, result):
-        if conn != self._conn:
-            conn.destroy ()
-            return
-
-        conn.destroy ()
-        self._conn = None
-        self._ppds = result
-        self._call_callback (None)
-
-    def _cups_error (self, conn, exc):
-        if conn != self._conn:
-            conn.destroy ()
-            return
-
-        conn.destroy ()
-        self._conn = None
-        self._ppds = None
-        self._call_callback (exc)
-
-    def _call_callback (self, exc):
-        if self._callback:
-            self._callback (self, exc)
+            self.emit ('finished')
 
+gobject.type_register(PPDsLoader)
 
 if __name__ == "__main__":
     class Foo:
@@ -240,12 +294,15 @@ if __name__ == "__main__":
             self._window = w
 
         def go (self, button):
-            PPDsLoader (self.ppds_loaded, device_id="MFG:MFG;MDL:MDL;",
-                        parent=self._window)
+            loader = PPDsLoader (device_id="MFG:MFG;MDL:MDL;",
+                                 parent=self._window)
+            loader.connect ('finished', self.ppds_loaded)
+            loader.run ()
 
-        def ppds_loaded (self, ppdsloader, exc):
+        def ppds_loaded (self, ppdsloader):
             self._window.destroy ()
             gtk.main_quit ()
+            exc = ppdsloader.get_error ()
             print exc
             ppds = ppdsloader.get_ppds ()
             if ppds != None:
